from .ggs_stuff import keeping
from .just_funcs import fakescanning, getserver
from .getconfig import getconfig
from .pstorm import ggs_login
__all__ = ["ggs_login", "fakescanning", "getconfig", "keeping", "getserver"]
__version__ = "1.2.0"



from configparser import ConfigParser
import json, os
def getconfig(type: str, savename: str, created: str | None = "y") -> dict:
    if created == "y":
        config = ConfigParser()
        # Use the user’s home directory and create the config file path
        home_directory = os.path.expanduser("~")
        configname = os.path.join(home_directory, type + "config.ini")
        config.read(configname)
        if savename not in config:
            raise ValueError(f"Config '{savename}' not found.")
        
        
        if savename in config:
            if type == "baron":
                nick = config.get(savename, "nick")
                pwrd = config.get(savename, "pwrd")
                server = config.get(savename, "server")
                kid = config.get(savename, "kid")
                excluded_commanders = config.get(savename, "excluded_commanders")
                radius_option = config.get(savename, "radius_option")
                distance = config.getfloat(savename, "distance")  # converted to float
                vip_option = config.get(savename, "vip_option")
                max_flank = config.getint(savename, "max_flank")  # converted to int
                max_front = config.getint(savename, "max_front")  # converted to int
                unit_id = config.get(savename, "unit_id")
                flank_id = config.get(savename, "flank_id")
                flank_tool_ammount = config.getint(savename, "flank_tool_ammount")  # converted to int
                front_id_1 = config.get(savename, "front_id_1")
                front_tool_ammount1 = config.getint(savename, "front_tool_ammount1")  # converted to int
                front_id_2 = config.get(savename, "front_id_2")
                front_tool_ammount2 = config.getint(savename, "front_tool_ammount2")  # converted to int
                # Return as a dictionary
                result = {
                    "nick": nick,
                    "pwrd": pwrd,
                    "server": server,
                    "kid": kid,
                    "excluded_commanders": excluded_commanders,
                    "radius_option": radius_option,
                    "distance": distance,
                    "vip_option": vip_option,
                    "max_flank": max_flank,
                    "max_front": max_front,
                    "unit_id": unit_id,
                    "flank_id": flank_id,
                    "flank_tool_ammount": flank_tool_ammount,
                    "front_id_1": front_id_1,
                    "front_tool_ammount1": front_tool_ammount1,
                    "front_id_2": front_id_2,
                    "front_tool_ammount2": front_tool_ammount2
                }
                
                # Convert to JSON string and return
                return result  # Return a formatted JSON string
    if created == "n":
        home_directory = os.path.expanduser("~")
        configname = os.path.join(home_directory, type + "config.ini")
        config_maker_barons(savename, configname)
    else:
        print("Invalid str as the created parameter. Use 'y' or 'n' - y if you have created the config already, 'n' if no.")
        exit()
        return {}
    return {}

def config_maker_barons(savename, filename: str):
    config = ConfigParser()
    unit_options = {
        "1": ("Distance Samurai", '35'),
        "2": ("Distance Veteran Demon", '10'),
        "3": ("Distance Mead lvl 10", '216'),
        "4": ("Distance Mead lvl 2", '207'),
        "5": ("Distance Mead lvl 1", '206'),
        "6": ("Distance Mead lvl 0", '205'),
        "7": ("Meelee   Mead lvl 10", '215'),
        "8": ("Meelee   Mead lvl 3", '198'),
        "9": ("Meelee   Mead lvl 2", '197'),
        "10": ("Meelee   Mead lvl 1", '196'),
        "11": ("Meelee   Mead lvl 0", '195')
    }

    flank_tool_options = {
        "1": ("20%  towers", '649'),
        "2": ("5%   ladders", '614'),
        "3": ("15%  steel walls (anti distance)", '651'),
        "4": ("5%   wooden walls (anti distance)", '651'),
        "5": ("--   None", '-1')
    }

    front_tool_options_1 = {
        "1": ("20%  towers", '649'),
        "2": ("5%   ladders", '614'),
        "3": ("15%  steel walls (anti distance)", '651'),
        "4": ("5%   wooden walls (anti distance)", '651'),
        "5": ("--   None", '-1')
    }

    front_tool_options_2 = {
        "1": ("20%  ram", '648'),
        "2": ("5%   ram", '611'),
        "3": ("--   None", '-1')
    }

    config.read(filename)
    print(f'Saves: {config.sections()}')
    save_name = savename
    while True:
        if not save_name:
            print("...")
            exit()
        elif config.has_section(save_name):
            if_overwrite = input("Save with this name already exists. Do you want to overwrite it?\n(y/n): ")
            if if_overwrite == 'y': 
                config.remove_section(save_name)
            else:
                print("Exitting.")
                exit()
        else:
            break
    config.add_section(save_name)

    def input_int_list(prompt):
        while True:
            raw = input(prompt).strip()
            if raw:
                try:
                    values = [int(item.strip()) for item in raw.split(",") if item.strip()]
                    return ",".join(str(val) for val in values)
                except ValueError:
                    print("Please enter valid comma separated integers (e.g. 2,29,30,31).")
            else:
                print("Input cannot be empty.")

    def input_int(prompt):
        while True:
            raw = input(prompt).strip()
            if raw:
                try:
                    return str(int(raw))
                except ValueError:
                    print("Please enter a valid integer.")
            else:
                print("Input cannot be empty.")

    nick = input("Enter your nickname: ")
    config.set(save_name, "nick", nick)
    passwd = input("Enter your password (stored local): ")
    config.set(save_name, "pwrd", passwd)
    server = input("Enter your server (e.g. PL1): ")
    config.set(save_name, "server", server)
    kid = input("Kingdom Green = g, Kingdom Fire = f, Kingdom Sand = s, Kingdom Cold = c\nKingdom -> <-\b\b\b")
    config.set(save_name, "kid", kid)

    excluded_commanders = input_int_list("Enter excluded commanders (comma separated integers, e.g. 2,3,17. -1 if none): ")
    config.set(save_name, "excluded_commanders", excluded_commanders)
    
    radius_option = "c"
    config.set(save_name, "radius_option", radius_option)

    distance = input_int("Distance for attacks (not preicse): ")
    config.set(save_name, "distance", distance)

    vip_option = input("Do you want the bot to use VIP commanders? (if no, it will send all commanders, wait until they come back and do it until the end.)\n(y/n) -> <-\b\b\b")
    if vip_option in ["y", "n"]:
        config.set(save_name, "vip_option", vip_option)
    else:
        print("Not an answer. y for yes, n for no.")
        exit()

    print("The script will use 4 waves. Always. Each wave will have the same setup.")
    max_flank = input_int("Enter ammount of units on a flank (0 if none)  : ")
    config.set(save_name, "max_flank", max_flank)

    max_front = input_int("Enter ammount of units on the front (0 if none): ")
    config.set(save_name, "max_front", max_front)

    print("\nPick units to send in the attack\n")
    for key, value in unit_options.items():
        print(f"{key}   - {value[0]}\n")
    unit_choice = (input("Selection: "))
    print(f'Selected option "{unit_options[unit_choice][0]}"')
    config.set(save_name, "unit_id", str(unit_options[unit_choice][1]))

    print("\nPick wich tool do you want to use on the flanks\n")
    for key, value in flank_tool_options.items():
        print(f"{key}   - {value[0]}\n")
    flank_choice = input("Selection: ")
    print(f'Selected option "{flank_tool_options[flank_choice][0]}"')
    config.set(save_name, "flank_id", str(flank_tool_options[flank_choice][1]))

    if flank_tool_options[flank_choice][0] != "--   None":
        flank_tool_ammount = input_int("Enter ammount of those tools per flank: ")
        config.set(save_name, "flank_tool_ammount", flank_tool_ammount)
    else:
        config.set(save_name, "flank_tool_ammount", "0")

    print("\nPick the first tool you want to use on the front\n")
    for key, value in front_tool_options_1.items():
        print(f"{key}   - {value[0]}\n")
    front_choice1 = input("Selection: ")
    print(f'Selected option "{front_tool_options_1[front_choice1][0]}"')
    config.set(save_name, "front_id_1", str(front_tool_options_1[front_choice1][1]))

    if front_tool_options_1[front_choice1][0] != "--   None":
        front_tool_ammount1 = input_int("Enter ammount of those tools per front: ")
        config.set(save_name, "front_tool_ammount1", front_tool_ammount1)
    else:
        config.set(save_name, "front_tool_ammount1", "0")

    print("\nPick the second tool you want to use on the front\n")
    for key, value in front_tool_options_2.items():
        print(f"{key}   - {value[0]}\n")
    front_choice2 = input("Selection: ")
    print(f'Selected option "{front_tool_options_2[front_choice2][0]}"')
    config.set(save_name, "front_id_2", str(front_tool_options_2[front_choice2][1]))

    if front_tool_options_2[front_choice2][0] != "--   None":
        front_tool_ammount2 = input_int("Enter ammount of those tools per front: ")
        config.set(save_name, "front_tool_ammount2", front_tool_ammount2)
    else:
        config.set(save_name, "front_tool_ammount2", "0")

    with open(filename, "w") as f:
        config.write(f)
    
    print(f"\nConfiguration saved.")
    exit()



import websockets, asyncio, random, re
import json as json_module
from .just_funcs import getserver
async def keeping(ws, server: str) -> None:
    """Keep the connection alive by sending periodic messages."""
    while ws.open:
        try:
            await ws.send(f"%xt%{server}%pin%1%<RoundHouseKick>%")
            await asyncio.sleep(60)  # Keep-alive interval
        except websockets.exceptions.ConnectionClosedError:
            print("Connection closed, stopping keep-alive")
            break



import asyncio
from concurrent.futures import ThreadPoolExecutor
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from .just_funcs import getserver
import json as json_module
import re
class RecaptchaTokenGenerator:
    def __init__(self):
        self.executor = ThreadPoolExecutor(max_workers=1)

    def _generate_recaptcha_token_sync(self):
        try:
            options = webdriver.ChromeOptions()
            options.add_argument("--log-level=3")
            options.add_argument("--no-sandbox")
            options.add_argument("--window-size=1,1")
            options.add_experimental_option("excludeSwitches", ["enable-automation"])
            driver = webdriver.Chrome(options=options)

            driver.get("https://empire.goodgamestudios.com/")
            wait = WebDriverWait(driver, 30, poll_frequency=0.01)
            wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'iframe#game')))
            iframe = driver.find_element(By.CSS_SELECTOR, 'iframe#game')
            driver.switch_to.frame(iframe)
            wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, '.grecaptcha-badge')))

            result = driver.execute_script("""
                return new Promise((resolve) => {
                    window.grecaptcha.ready(() => {
                        window.grecaptcha.execute('6Lc7w34oAAAAAFKhfmln41m96VQm4MNqEdpCYm-k', { action: 'submit' }).then(resolve);
                    });
                });
            """)

            driver.quit()
            return result
        except Exception as e:
            raise e

    async def generate_recaptcha_token(self):
        loop = asyncio.get_running_loop()
        return await loop.run_in_executor(self.executor, self._generate_recaptcha_token_sync)

async def ggs_login(ws, nick: str, pwrd: str, server: str, kid: int | None = 0) -> tuple:
    
    """
    Login to your account and return the coordinates of your main castle.
    """
    server = getserver(server, "ex")
    generator = RecaptchaTokenGenerator()
    token = await generator.generate_recaptcha_token()
    if ws.open:
        await ws.send(f"""<msg t='sys'><body action='verChk' r='0'><ver v='166' /></body></msg>""")
        await ws.send(f"""<msg t='sys'><body action='login' r='0'><login z='{server}'><nick><![CDATA[]]></nick><pword><![CDATA[605015%pl%0]]></pword></login></body></msg>""")
        await ws.send(f"""<msg t='sys'><body action='autoJoin' r='-1'></body></msg>""")
        await ws.send(f"""<msg t='sys'><body action='roundTrip' r='1'></body></msg>""")
        await ws.send(f"""%xt%{server}%vln%1%{{"NOM": "{nick}"}}%""")
        await ws.send(f"""%xt%{server}%lli%1%{{"CONM":1752,"RTM":24,"ID":0,"PL":1,"NOM":"{nick}","PW":"{pwrd}","LT":null,"LANG":"pl","DID":"0","AID":"1748087142659830366","KID":"","REF":"https://empire.goodgamestudios.com","GCI":"","SID":9,"PLFID":1,"RCT":"{token}"}}%""")
        while ws.open:
            try:
                response = await asyncio.wait_for(ws.recv(), timeout=7.5)
                response = response.decode('utf-8')

                if "%xt%lli%1%" in response:
                    if "%xt%lli%1%0%" not in response:
                        print("Wrong login data.")
                        exit()
                elif "%xt%gbd%1%0%" in response:
                    response = response.replace('%xt%gbd%1%0%', '').rstrip('%').strip()
                    response = json_module.loads(response)
                    lids = []
                    fragments = response["gli"]["C"]
                    for fragment in fragments:
                        lids.append(fragment["ID"])
                    lids = sorted(lids)
                    break
            except asyncio.TimeoutError:
                break
        await ws.send(f"%xt%{server}%nch%1%")
        await ws.send(f"""%xt%{server}%core_gic%1%{{"T":"link","CC":"PL","RR":"html5"}}%""")
        await ws.send(f"%xt%{server}%gbl%1%{{}}%")
        await ws.send(f"""%xt%{server}%jca%1%{{"CID":-1,"KID":0}}%""")
        await ws.send(f"%xt%{server}%alb%1%{{}}%")
        await ws.send(f"%xt%{server}%sli%1%{{}}%")
        await ws.send(f"%xt%{server}%gie%1%{{}}%")
        await ws.send(f"%xt%{server}%asc%1%{{}}%")
        await ws.send(f"%xt%{server}%sie%1%{{}}%")
        await ws.send(f"""%xt%{server}%ffi%1%{{"FIDS":[1]}}%""")
        await ws.send(f"%xt%{server}%kli%1%{{}}%")
        while ws.open:
            try:
                response = await asyncio.wait_for(ws.recv(), timeout=6)
                response = response.decode('utf-8')
                if "%xt%jaa%1%0%" in response:
                    sx_list = []
                    sy_list = []
                    cid_list = []
                    for i in range(4):
                        pattern = rf"\[{i},(\d+),(\d+),(\d+),1"
                        match = re.search(pattern, response)
                        if match:
                            cid = match.group(1)
                            sx = match.group(2)
                            sy = match.group(3)
                            sx_list.append(sx)
                            sy_list.append(sy)
                            cid_list.append(cid)
                            print(f"Coord {i} X: {sx}, Coord Y: {sy}")
                    break

            except asyncio.TimeoutError:
                break

        while ws.open:
            try:
                response = await asyncio.wait_for(ws.recv(), timeout=5.5)
                response = response.decode('utf-8')
                if "%xt%ffi%1%0%" in response:
                    await ws.send(f"%xt%{server}%gcs%1%{{}}%")
                    print("Successfully logged in")
                    break
            except asyncio.TimeoutError:
                break
    else:
        print("Connection closed, stopping login")
    return sx_list, sy_list, lids, cid_list


import asyncio, re
import json as json_module
from importlib.resources import files
from importlib.metadata import version as get_installed_version, PackageNotFoundError
async def c2s_search_for(ws, c2s_code: str, waiting_time: float | None = 3) -> dict | int:
    """Phrase is made from 3 letters. Returns response json or error code."""
    while True:
        try:
            response = await asyncio.wait_for(ws.recv(), timeout=waiting_time)
            response = response.decode('utf-8')
            phrase = rf'%xt%{c2s_code}%1%(\d+)%'
            starting_info = re.search(phrase, response)
            if starting_info is not None:
                error_code = starting_info.group(1)
                if error_code == "0":
                    response = response.replace(f'%xt%{c2s_code}%1%0%', '').rstrip('%').strip()
                    return json_module.loads(response)
                else:
                    return int(error_code)
        except asyncio.TimeoutError:
            return -1

def getserver(server: str, option: str | None = "ex") -> str:
    ''' Get the server URI and server prefix from the server list. 

    :option: 
    :ex:  returns server prefix only
    :ws:  returns websocket uri only
    '''
    data_path = files("gjt").joinpath("server_list.json")
    with data_path.open("r", encoding="utf-8") as f:
        data = json_module.load(f)
        wsuri = data["servers"][server]["wsuri"]
        exname = data["servers"][server]["exname"]
        fulldata = {"wsuri": wsuri, "exname": exname}
        if option == "ex":
            return exname
        elif option == "ws":  
            return wsuri
        else:
            raise ValueError("Invalid option. Use 'full', 'ex', or 'ws'.")
        
async def fakescanning(ws, server: str, kid: str | None = "0") -> None:
    """
    Fake scanning the map while doing other things
    
    """
    empireex = getserver(server, "ex")
    delays = [6, 2, 4, 2]
    while ws.open:
        for delay in delays:
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":0,"AY1":0,"AX2":12,"AY2":12}}%""")
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":1274,"AY1":0,"AX2":1286,"AY2":12}}%""")
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":13,"AY1":0,"AX2":25,"AY2":12}}%""")
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":1274,"AY1":13,"AX2":1286,"AY2":25}}%""")
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":0,"AY1":13,"AX2":12,"AY2":25}}%""")
            await ws.send(f"""%xt%{empireex}%gaa%1%{{"{kid}":0,"AX1":13,"AY1":13,"AX2":25,"AY2":25}}%""")
            await asyncio.sleep(delay * 60) 
